package csci1902.hw3;

import java.awt.Point;

/**
 * Class implementing a food source for ants.
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 *
 */
public class FoodSource extends Entity {
    private int foodUnits = 0;
    private int nextFood = 0;
    
    /**
     * Constructs a new food source
     * @param w The simulation world
     * @param location The food source location
     * @param quantity The amount of food present at the food source
     */
    public FoodSource(World w, Point location, int quantity) { 
        super(w, location);
        foodUnits = quantity;
        nextFood = quantity;
    }
    
    public void update() {
        super.update();
        foodUnits = nextFood;
    }
    /**
     * Query the amount of food remaining at the food source.
     * @return The amount of food remaining at the food source.
     */
    public int getFoodUnits() {
        return foodUnits;
    }
    
    /**
     * Query whether the entity is a food source.
     * @return true
     */
    public boolean isFoodSource() {
        return true;
    }
    
    /**
     * Collect some food from the food source.  This change is applicable in
     * the next state.
     * @param amount The amount of food to attempt to collect.
     * @return The amount of food collected (can be less than {@code amount} if
     * the food source is running out of food).
     */
    public int collectFood(int amount) {
        if (amount > nextFood) {
            amount = nextFood;
            nextFood = 0;
            die();
        } else {
            nextFood -= amount;
        }
        return amount;
    }    
}
