package csci1902.hw3;

import junit.framework.TestCase;
import java.awt.Point;
import java.util.*;

/**
 * Tests the World class.
 * @author Michael Ekstrand <ekstrand@cs.umn.edu>
 */
public class WorldTest extends TestCase {
    World world;
    
    /**
     * Start each test with a world of size 10x10 with the colony at 5,5.
     */
    protected void setUp() {
        world = new World(10, 10, new Point(5,5));
    }
    
    /**
     * Tests the initial dimensions of a world.
     */
    public void testDimensions() {
        assertEquals(10, world.getHeight());
        assertEquals(10, world.getWidth());
        
        world = new World(5, 10);
        assertEquals(5, world.getWidth());
        assertEquals(10, world.getHeight());
    }
    
    /**
     * Tests the initial state of a world.
     */
    public void testInitialState() {
        // test for 0 pheromone levels
        for (int i = 0; i < world.getWidth(); i++) {
            for (int j = 0; j < world.getHeight(); j++) {
                assertEquals(0, world.getFoodPheromone(i, j), 0.0001);
            }
        }
        // make sure we have a colony
        assertNotNull(world.getColony());
        assertEquals(new Point(5,5), world.getColony().getPosition());
        
        // make sure the entity list contains the colony
        List<Entity> ents = world.getEntities();
        assertNotNull(ents);
        assertEquals(1, ents.size());
        assertTrue(ents.get(0) instanceof Colony);
        assertSame(ents.get(0), world.getColony());
    }
    
    /**
     * Tests the food pheromone deposit functionality.
     */
    public void testDepositFood() {
        world.depositFoodPheromone(1, 1, 15);
        // make sure it hasn't taken effect yet
        assertEquals(0, world.getFoodPheromone(1, 1), 0.0001);
        // commit and make sure the pheromone is present
        world.commit();
        assertEquals(15, world.getFoodPheromone(1, 1), 0.0001);
    }
    
    /**
     * Tests that pheromone levels decay over time.
     */
    public void testFoodDecay() {
        world.depositFoodPheromone(1, 1, 15);
        world.commit();
        assertEquals(15, world.getFoodPheromone(1, 1), 0.0001);
        world.commit();
        // 13.05826 is 15 decayed 1 time step with a half-life of 5
        assertEquals(13.05826, world.getFoodPheromone(1, 1), 0.0001);
    }
    
    /**
     * Test the functionality of adding an ant.  It makes sure that the ant
     * shows up in the entities list.
     */
    public void testAddAnt() {
        world.addForagerAnt();
        List<Entity> ents = world.getEntities();
        assertEquals(2, ents.size());
        Entity ant = ents.get(1);
        assertNotNull(ant);
        assertEquals(new Point(5,5), ant.getPosition());
        
        assertTrue(ant instanceof ForagerAnt);
        assertEquals(new Point(5,5), ((Ant) ant).getHomePosition());
    }
}
